<?php
/*
Plugin Name: Author Posts
plugin URI: https://99robots.com/?post_type=download&p=8333
Description: Showcase your author's posts with beautiful customizable layouts.
version: 1.2.1
Author: 99 Robots
Author URI: https://99robots.com
License: GPL2
*/

// Exit if accessed directly

if ( !defined( 'ABSPATH' ) ) exit;

/* Item Name */

if (!defined('NNROBOTS_POST_AUTHOR_ITEM_NAME'))
    define('NNROBOTS_POST_AUTHOR_ITEM_NAME', 'Author Posts');

/* Store URL */

if (!defined('NNROBOTS_POST_AUTHOR_STORE_URL'))
    define('NNROBOTS_POST_AUTHOR_STORE_URL', 'https://99robots.com');

/* Minimum PHP version */

if (!defined('NNROBOTS_POST_AUTHOR_MIN_PHP_VERSION'))
    define('NNROBOTS_POST_AUTHOR_MIN_PHP_VERSION', '5.4.0');

/* Plugin Name */

if (!defined('NNROBOTS_POST_AUTHOR_PLUGIN_NAME'))
    define('NNROBOTS_POST_AUTHOR_PLUGIN_NAME', trim(dirname(plugin_basename(__FILE__)), '/'));

/* Plugin directory */

if (!defined('NNROBOTS_POST_AUTHOR_PLUGIN_DIR'))
    define('NNROBOTS_POST_AUTHOR_PLUGIN_DIR', plugin_dir_path(__FILE__) );

/* Plugin url */

if (!defined('NNROBOTS_POST_AUTHOR_PLUGIN_URL'))
    define('NNROBOTS_POST_AUTHOR_PLUGIN_URL', plugins_url() . '/' . NNROBOTS_POST_AUTHOR_PLUGIN_NAME);

/* Plugin verison */

if (!defined('NNROBOTS_POST_AUTHOR_VERSION_NUM'))
    define('NNROBOTS_POST_AUTHOR_VERSION_NUM', '1.2.1');

/**
 * Activatation / Deactivation
 */

register_activation_hook( __FILE__, 			array('NNRobots_Post_Author', 'register_activation'));


/**
 * Hooks / Filter
 */

add_action('admin_menu', 						array('NNRobots_Post_Author', 'menu'));
add_action('init', 								array('NNRobots_Post_Author', 'load_textdoamin'));
$plugin = plugin_basename(__FILE__);
add_filter("plugin_action_links_$plugin", 		array('NNRobots_Post_Author', 'settings_link'));

// Add Shortcode

add_action('media_buttons', 					array('NNRobots_Post_Author', 'add_media_buttons'), 20);
add_action('wp_enqueue_media', 					array('NNRobots_Post_Author', 'include_shortcode_button_scripts'));
add_shortcode('nnr_author_posts',				'nnr_author_posts_shortcode');

// Include Widgets

require_once('author-posts-widget.php');
require_once('featured-author-widget.php');

// Software Licensing Updates

add_action('wp_ajax_nnr_ap_license_action', 	array('NNRobots_Post_Author', 'license_action'));

if ( !class_exists('EDD_SL_Plugin_Updater') ) {
	include_once( dirname( __FILE__ ) . '/admin/include/edd-software-licensing-updates/EDD_SL_Plugin_Updater.php' );
}

$edd_updater = new EDD_SL_Plugin_Updater( NNROBOTS_POST_AUTHOR_STORE_URL , __FILE__, array(
		'version' 	=> NNROBOTS_POST_AUTHOR_VERSION_NUM,
		'license' 	=> NNRobots_Post_Author::get_license_key(),
		'item_name' => NNROBOTS_POST_AUTHOR_ITEM_NAME,
		'author' 	=> '99 Robots',
	)
);

/**
 * NNRobots_Post_Author class.
 *
 * @extends WP_Widget
 */
class NNRobots_Post_Author {

	/**
	 * prefix
	 *
	 * (default value: 'nnrobots_post_by_author_')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $prefix = 'nnrobots_post_by_author_';

	/**
	 * prefix_dash
	 *
	 * (default value: 'nnr-ap-')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $prefix_dash = 'nnr-ap-';

	/**
	 * updates_page
	 *
	 * (default value: 'nnrobots-author-posts-updates')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $settings_page = 'nnrobots-author-posts-settings';

	/**
	 * text_domain
	 *
	 * (default value: 'nnrobots-author-posts')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $text_domain = '99robots-author-posts';

	/**
	 * register_activation function.
	 * Performs tasks needed upon activation
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function register_activation() {

		/* Check if multisite, if so then save as site option */

		if (is_multisite()) {
			add_site_option(self::$prefix . 'version', NNROBOTS_POST_AUTHOR_VERSION_NUM);
		} else {
			add_option(self::$prefix . 'version', NNROBOTS_POST_AUTHOR_VERSION_NUM);
		}
	}

	/**
	 * load_textdoamin function.
	 * Load the text domain
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function load_textdoamin() {
		load_plugin_textdomain(self::$text_domain, false, basename( dirname( __FILE__ ) ) . '/languages' );
	}

	/**
	 * Hooks to 'plugin_action_links_' filter
	 *
	 * @since 1.0.0
	 */
	static function settings_link($links) {
		$settings_link = '<a href="options-general.php?page=' . self::$settings_page . '">Settings</a>';
		array_unshift($links, $settings_link);

		$widgets_link = '<a href="widgets.php">Widget</a>';
		array_unshift($links, $widgets_link);

		return $links;
	}

	/**
	 * Perform all the actions with a license
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function license_action() {

		if ( !isset($_POST['license_key']) ) {
			echo '';
			die();
		}

		// data to send in our API request

		$api_params = array(
			'edd_action'	=> $_POST['license_action'],
			'license' 		=> $_POST['license_key'],
			'item_name' 	=> urlencode( NNROBOTS_POST_AUTHOR_ITEM_NAME ), // the name of our product in EDD
			'url'       	=> is_multisite() ? network_home_url() : home_url()
		);

		// Call the custom API.

		$response = wp_remote_post( NNROBOTS_POST_AUTHOR_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// make sure the response came back okay

		if ( is_wp_error( $response ) ) {
			echo $response;
			die();
		}

		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		if ( $_POST['license_action'] == 'activate_license' ) {
			self::update_license_key($_POST['license_key']);
		} else if ( $_POST['license_action'] == 'deactivate_license' && $license_data->license == 'deactivated') {
			self::update_license_key('');
		}

		// decode the license data

		echo $license_data->license;

		die();

	}

	/**
	 * Check if license key is active
	 *
	 * @access public
	 * @static
	 * @param mixed $license_key
	 * @return void
	 */
	static function is_license_active( $license_key ) {

		if ( !isset( $license_key ) ) {
			return false;
		}

		// data to send in our API request

		$api_params = array(
			'edd_action'	=> 'check_license',
			'license' 		=> $license_key,
			'item_name' 	=> urlencode( NNROBOTS_POST_AUTHOR_ITEM_NAME ), // the name of our product in EDD
			'url'       	=> is_multisite() ? network_home_url() : home_url()
		);

		// Call the custom API.

		$response = wp_remote_post( NNROBOTS_POST_AUTHOR_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// make sure the response came back okay

		if ( is_wp_error( $response ) ) {
			return false;
		}

		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		// decode the license data

		if ( isset($license_data->license) && $license_data->license == 'valid' ) {
			return true;
		} else {
			return false;
		}

	}

	/**
	 * Get the license key
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function get_license_key() {

		// Get the license key from based on WordPress install

		if ( function_exists('is_multisite') && is_multisite() ) {
			$license_key = get_site_option(self::$prefix . 'license_key');
		} else {
			$license_key = get_option(self::$prefix . 'license_key');
		}

		// Check if license key is set

		if ( $license_key === false ) {
			$license_key = '';
		}

		return $license_key;
	}

	/**
	 * Update the license key
	 *
	 * @access public
	 * @static
	 * @param mixed $license_key
	 * @return void
	 */
	static function update_license_key($license_key) {

		if ( isset( $license_key ) ) {

			// Get the license key from based on WordPress install

			if ( function_exists('is_multisite') && is_multisite() ) {

				update_site_option(self::$prefix . 'license_key', $license_key);

			} else {

				update_option(self::$prefix . 'license_key', $license_key);

			}
		}
	}

	/**
	 * Add a media button for adding Optin Fire shortcode
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function add_media_buttons(){

		echo '<button class="button ' . self::$prefix_dash . 'shortcode-builder"><span></span>' . __('Author Posts', self::$text_domain) . '</button>
		<div class="' . self::$prefix_dash . 'shortcode-builder-container">
		<div class="postbox ' . self::$prefix_dash . 'shortcode-builder-box" style="display:none;">
			<h3>
				<span>' . __('Author Posts Shortcode Builder', self::$text_domain) . '</span>

				<button style="float: right;margin-right: 0px;" class="button ' . self::$prefix_dash . 'shortcode-builder-close">' . __('Close', self::$text_domain) . '</button>
				<button style="float: right;margin-right: 12px;" class="button button-primary ' . self::$prefix_dash . 'shortcode-builder-add">' . __('Insert Shortcode', self::$text_domain) . '</button>
			</h3>
			<div class="inside">
				<p style="display:none;">
					<strong><label for="' . self::$prefix_dash . 'shortcode">' . __( 'ShortCode:', self::$text_domain). '</label></strong>
					<code class="' . self::$prefix_dash . 'shortcode">[nnr_author_posts]</code>
				</p>

				<!-- Title -->

				<p>
					<label for="' . self::$prefix_dash . 'title">' . __('Title:', self::$text_domain) . '</label>
					<input id="' . self::$prefix_dash . 'title" name="' . self::$prefix_dash . 'title" type="text" value="' . __('Author Posts', self::$text_domain) . '" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="title"/>
				</p>

				<!-- Author -->

				<p>
					<label for="' . self::$prefix_dash . 'author">' . __('Author:', self::$text_domain) . '</label>
					<input id="' . self::$prefix_dash . 'author" name="' . self::$prefix_dash . 'author" type="text" value="0" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="author"/>
				</p>

				<!-- Number Posts -->

				<p>
					<label>' . __('Number of Posts to display:' ,self::$text_domain) . '</label>
					<select id="' . self::$prefix_dash . 'num-posts" name="' . self::$prefix_dash . 'num-posts" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="num_posts">
						<option value="all" selected="selected">' . __('All', self::$text_domain) . '</option>';

						for ($i = 1; $i < 21; $i++) {
							echo '<option value="' . $i . '">' . __($i, self::$text_domain) . '</option>';
						}

					echo '</select>
				</p>

				<!-- Sort -->

				<p>
					<label>' . __('Sort Posts by:' ,self::$text_domain) . '</label>
					<select id="' . self::$prefix_dash . 'orderby" name="' . self::$prefix_dash . 'orderby" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="orderby">
						<option value="ID" selected="selected">' . __('ID', self::$text_domain) . '</option>
						<option value="title">' . __('Post Title', self::$text_domain) . '</option>
						<option value="date">' . __('Post Date', self::$text_domain) . '</option>
						<option value="comment_count">' . __('Comment Count', self::$text_domain) . '</option>
					</select>
				</p>

				<!-- Order -->

				<p>
					<label>' . __('Order Posts by:' ,self::$text_domain) . '</label>
					<select id="' . self::$prefix_dash . 'order" name="' . self::$prefix_dash . 'order" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="order">
						<option value="ASC" selected="selected">' . __('ASC', self::$text_domain) . '</option>
						<option value="DESC">' . __('DESC', self::$text_domain) . '</option>
					</select>
				</p>

				<!-- Avatar Layout -->

				<p>
					<label>' . __('Author Layout:', self::$text_domain) . '</label>
					<select id="' . self::$prefix_dash . 'author-layout" name="' . self::$prefix_dash . 'author-layout" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="author_layout">
						<option value="inline" selected="selected">' . __('Inline', self::$text_domain) . '</option>
						<option value="stacked">' . __('Stacked', self::$text_domain) . '</option>
					</select>
				</p>

				<!-- Author Bio -->

				<p>
					<input type="checkbox" class="' . self::$prefix_dash . 'author-description ' . self::$prefix_dash . 'shortcode-input" name="' . self::$prefix_dash . 'author-description" data-shortcode="author_description"/>
					<label>' . __('Author Biography', self::$text_domain) . '</label>
				</p>

				<!-- Avatar Shape -->

				<p>
					<label>' . __('Avatar Shape:', self::$text_domain) . '</label>
					<select id="' . self::$prefix_dash . 'avatar-shape" name="' . self::$prefix_dash . 'avatar-shape" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="avatar_shape">
						<option value="square" selected="selected">' . __('Square', self::$text_domain) . '</option>
						<option value="rounded-square">' . __('Rounded Square', self::$text_domain) . '</option>
						<option value="circle">' . __('Circle', self::$text_domain) . '</option>
					</select>
				</p>

				<!-- Avatar Size -->

				<p>
					<label>' . __('Avatar Size:', self::$text_domain) . '</label>
					<select id="' . self::$prefix_dash . 'avatar-size" name="' . self::$prefix_dash . 'avatar-size" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="avatar_size">
						<option value="small" selected="selected">' . __('Small', self::$text_domain) . '</option>
						<option value="medium">' . __('Medium', self::$text_domain) . '</option>
						<option value="large">' . __('Large', self::$text_domain) . '</option>
						<option value="full">' . __('Full', self::$text_domain) . '</option>
					</select>
				</p>

				<!-- Posts Layout -->

				<p>
					<label>' . __('Post Layout:', self::$text_domain) . '</label>
					<select id="' . self::$prefix_dash . 'post_layout" class="' . self::$prefix_dash . 'layout ' . self::$prefix_dash . 'shortcode-input" name="' . self::$prefix_dash . 'post_layout" data-shortcode="post_layout">
						<option value="list" selected="selected">' . __('List', self::$text_domain) . '</option>
						<option value="list_excerpt">' . __('List with Excerpt', self::$text_domain) . '</option>
						<option value="thumbnail">' . __('Thumbnail', self::$text_domain) . '</option>
						<option value="numbered_list">' . __('Numbered List', self::$text_domain) . '</option>
						<option value="image_tiles">' . __('Image Tiles', self::$text_domain) . '</option>
						<option value="image_flip">' . __('Image Flip', self::$text_domain) . '</option>
						<option value="grid">' . __('Grid', self::$text_domain) . '</option>
						<option value="slider">' . __('Slider', self::$text_domain) . '</option>
						<option value="fancy_colors">' . __('Fancy Colors', self::$text_domain) . '</option>
						<option value="colored_tabs">' . __('Colored Tabs', self::$text_domain) . '</option>
					</select>
				</p>

				<!-- Grid Size -->

				<p class="' . self::$prefix_dash . 'grid-size">
					<label>' . __('Grid Size:', self::$text_domain) . '</label>
					<select id="' . self::$prefix_dash . 'grid-size" name="' . self::$prefix_dash . 'grid_size" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="grid_size">
						<option value="small" selected="selected">' . __('Small', self::$text_domain) . '</option>
						<option value="medium">' . __('Medium', self::$text_domain) . '</option>
						<option value="large">' . __('Large', self::$text_domain) . '</option>
					</select>
				</p>

				<!-- Fancy Colors -->

				<p class="' . self::$prefix_dash . 'fancy-colors">
					<strong><label for="' . self::$prefix_dash . 'fancy-colors">' . __('Fancy Colors:', self::$text_domain) . '</label></strong><br/>
					<input id="' . self::$prefix_dash . 'fancy-colors" name="' . self::$prefix_dash . 'fancy-colors" value="#D14841,#F37934,#FAC51C,#61BD6D,#2C82C9,#553982" class="' . self::$prefix_dash . 'fancy-colors-selectize ' . self::$prefix_dash . 'shortcode-input" style="display:none;" data-shortcode="fancy_colors"><br/>
					<em class="' . self::$prefix_dash . 'fancy-colors">' . __('The background color of each item.  Drag and drop to re-order colors.', self::$text_domain) . '</em>
				</p>

				<!-- Show Nav -->

				<p class="' . self::$prefix_dash . 'show-nav">
					<input type="checkbox" id="' . self::$prefix_dash . 'show-nav" name="' . self::$prefix_dash . 'show-nav" class="' . self::$prefix_dash . 'shortcode-input" data-shortcode="show_nav" checked="checked"/>
					<label>' . __('Show Navigation Arrows', self::$text_domain) . '</label>
				</p>

				<!-- Style Override -->

				<p>
					<input id="' . self::$prefix_dash . 'style-override" name="' . self::$prefix_dash . 'style-override" class="' . self::$prefix_dash . 'style-override ' . self::$prefix_dash . 'shortcode-input" type="checkbox" data-shortcode="style_override">
					<label for="' . self::$prefix_dash . 'style-override">' . __('Theme Style Override:', self::$text_domain) . '</label><br/>
					<em>' . __('Use your own custom styling for text and background, or your themes styling.', self::$text_domain) . '</em>
				</p>

				<!-- Color -->

				<p class="' . self::$prefix_dash . 'color">
					<strong><label for="' . self::$prefix_dash . 'color">' . __('Color:', self::$text_domain) . '</label></strong><br/>
					<input id="' . self::$prefix_dash . 'color" name="' . self::$prefix_dash . 'color" class="' . self::$prefix_dash . 'bg-color-picker ' . self::$prefix_dash . 'shortcode-input" value="#eee" data-shortcode="color"><br/>
					<em>' . __('The background color of the image tiles.', self::$text_domain) . '</em><br/>
					<em>' . __('Click inside textfield to bring up color picker.', self::$text_domain) . '</em>
				</p>

				<!-- Text Color -->

				<p class="' .self::$prefix_dash . 'text-color">
					<strong><label for="' . self::$prefix_dash . 'text-color">' . __('Text Color:', self::$text_domain) . '</label></strong><br/>
					<input id="' . self::$prefix_dash . 'text-color" name="' . self::$prefix_dash . 'text-color" class="' . self::$prefix_dash . 'color-picker ' . self::$prefix_dash . 'shortcode-input" value="#000000" data-shortcode="text_color"><br/>
					<em>' . __('The text color for each item.', self::$text_domain) . '</em><br/>
					<em>' . __('Click inside textfield to bring up color picker.', self::$text_domain) . '</em>
				</p>

				<!-- Display Asynchronously -->

				<p>
					<input type="checkbox" id="' . self::$prefix_dash . 'async" name="' . self::$prefix_dash . 'async" data-shortcode="async" class="' . self::$prefix_dash . 'shortcode-input"/>
					<label>' . __('Display Asynchronously', self::$text_domain) . '</label>
				</p>
			</div>
		</div>
		</div>';

	}

	/**
	 * Include the shortcode builder stylesheet and script
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function include_shortcode_button_scripts() {

		if ( isset($_GET['post']) ) {

			// Styles

			wp_enqueue_style(self::$prefix . 'shortcode_builder_css', 		NNROBOTS_POST_AUTHOR_PLUGIN_URL . '/admin/css/shortcode-builder.css');
			wp_enqueue_style('wp-color-picker');

			// Scripts


			wp_enqueue_script('wp-color-picker');
			wp_enqueue_style(self::$prefix . 'selectize_css', 				NNROBOTS_POST_AUTHOR_PLUGIN_URL . '/admin/css/selectize.css');
			wp_enqueue_script(self::$prefix . 'selectize_js', 				NNROBOTS_POST_AUTHOR_PLUGIN_URL . '/admin/js/selectize.min.js');
			wp_enqueue_script(self::$prefix . 'shortcode_builder_js', 		NNROBOTS_POST_AUTHOR_PLUGIN_URL . '/admin/js/shortcode-builder.js', array('jquery'));
			wp_localize_script(self::$prefix . 'shortcode_builder_js', 		'nnr_ap_shortcode', array(
				'prefix' 	=> self::$prefix_dash,
				'authors'	=> get_users(array('who' => 'authors', 'number' => 500)),
			));
		}

	}

	/**
	 * Hook for the admin menu
	 *
	 * @since 1.0.0
	 *
	 * @param 	N/A
	 * @return 	N/A
	 */
	static function menu() {

		// Settings page

		$settings_page_load = add_submenu_page(
	 		'options-general.php',
	 		__("Author Posts", self::$text_domain),
	 		__("Author Posts", self::$text_domain),
	 		'manage_options',
	 		self::$settings_page,
	 		array('NNRobots_Post_Author','settings'));

	 	add_action("admin_print_scripts-$settings_page_load", array('NNRobots_Post_Author', 'admin_include_scripts'));
	}

	/**
	 * include_scripts function.
	 *
	 * Include css on admin
	 * @access public
	 * @static
	 * @return void
	 */
	static function admin_include_scripts() {

		// Styles

		wp_enqueue_style(self::$prefix . 'settings_css', 	NNROBOTS_POST_AUTHOR_PLUGIN_URL . '/admin/css/settings.css');
		wp_enqueue_style(self::$prefix . 'bootstrap_css', 	NNROBOTS_POST_AUTHOR_PLUGIN_URL . '/admin/css/nnr-bootstrap.min.css');
		wp_enqueue_style(self::$prefix . 'fontawesome_css',	NNROBOTS_POST_AUTHOR_PLUGIN_URL . '/admin/include/fontawesome/css/font-awesome.min.css');

		// Scripts

		wp_enqueue_script(self::$prefix . 'bootstrap_js', 	NNROBOTS_POST_AUTHOR_PLUGIN_URL . '/admin/js/bootstrap.min.js', array('jquery'));
		wp_enqueue_script(self::$prefix . 'settings_js', 	NNROBOTS_POST_AUTHOR_PLUGIN_URL . '/admin/js/settings.js', array('jquery'));
		wp_localize_script(self::$prefix . 'settings_js', 	'nnr_ap_data', array(
			'prefix_dash'			=> self::$prefix_dash,
			'activate_text'			=> __('Activate', self::$text_domain),
			'deactivate_text'		=> __('Deactivate', self::$text_domain),
			'no_license_key'		=> __('Please add or activate your License Key to get automatic updates.  Without a valid license key you will not receive regular updates. You can find your license key', self::$text_domain) . ' <a href="https://99robots.com/dashboard" target="_blank">here</a>',
			'expired'				=> __('EXPIRED: This license key has expired.  Please renew your license key', self::$text_domain) . ' <a href="https://99robots.com/dashboard" target="_blank">here</a>',
			'activation_error'		=> __('INVALID LICENSE KEY: The license key is not valid please try again. You can find your license key', self::$text_domain) . ' <a href="https://99robots.com/dashboard" target="_blank">here</a>.',
			'activation_valid'		=> __('SUCCESS: Your license key is valid.', self::$text_domain),
			'deactivation_valid'	=> __('SUCCESS: This site has been deactivated.', self::$text_domain),
			'deactivation_error'	=> __('DEACTIVATION FAILED: This site could not be deactivated.', self::$text_domain),
		));
	}

	/**
	 * settings function.
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function settings() {

		global $wp_version;

		$license_key = self::get_license_key();

		require('admin/views/settings.php');

	}
}

/**
 * Create shortcode [nnr_author_posts ...]
 *
 * @since 1.0.0
 *
 * @param 	array of parameters
 * @return 	html code
 */
function nnr_author_posts_shortcode($atts) {

	$args = shortcode_atts( array(
		'title'               => __('Author Posts', NNRobots_Post_Author::$text_domain),
		'author'			  => 0,
		'orderby'             => 'ID',
		'order'               => 'ASC',
		'async'               => false,
		'num_posts'           => '5',
		'avatar_shape'        => 'ronuded-sqaure',
		'avatar_size'         => 'large',
		'author_layout'       => 'inline',
		'author_description'  => false,
		'post_layout'         => 'list',
		'grid_size'           => 'medium',
		'show_nav'            => true,
		'fancy_colors'        => '',
		'style_override'      => false,
		'color'               => '#efefef',
		'text_color'          => '#000000',
	), $atts );

	global $post;

	// Include slider files

	if ($args['post_layout'] == 'slider') {
		wp_enqueue_script(NNRobots_Post_Author::$prefix . 'slider_js', plugins_url('core/includes/flex-slider/jquery.flexslider-min.js', __FILE__), array('jquery'));
		wp_enqueue_style(NNRobots_Post_Author::$prefix . 'slider_css', plugins_url('core/includes/flex-slider/flexslider.css', __FILE__));
	}

	if ( !isset($_COOKIE[NNRobots_Post_Author::$prefix . 'shortcode_id']) ) {
		$_COOKIE[NNRobots_Post_Author::$prefix . 'shortcode_id'] = 0;
	}

	$_COOKIE[NNRobots_Post_Author::$prefix . 'shortcode_id']++;
	$this_id = NNRobots_Post_Author::$prefix_dash . 'shortcode-' . $_COOKIE[NNRobots_Post_Author::$prefix . 'shortcode_id'];

	$code = '';

	// AJAX

	if ( isset($args['async']) && $args['async'] ) {

		$code .= '<script type="text/javascript">
		jQuery(document).ready(function($) {

			var data = {
				"action": "nnr_ap_display",
				"widget_id" : "' . $this_id . '",
				"instance" : ' . json_encode($args) . ',
				"post": ' . json_encode($post) . ',
			};

			$.post("' . admin_url( 'admin-ajax.php' ) . '", data, function(response) {
				response = jQuery.parseJSON(response);
				$("#" + response.widget_id).append(response.content).hide().fadeIn();';

				if ($args['post_layout'] == 'slider') {
					$code .= '$("#" + response.widget_id + " .' . NNRobots_Post_Author::$prefix_dash . 'slider").flexslider({
						animation: "fade",';

					if (isset($args[ 'show_nav' ]) && $args[ 'show_nav' ]) {
						$code .= 'directionNav: true,';
					} else {
						$code .= 'directionNav: false,';
					}

					$code .= '});';
				}

			$code .= '});
		});
		</script>';
	} else {
		$code .= NNRobots_Post_Author_Widget::display_posts_by_author($args, $post, true, $args['author'], $this_id);
		$code .= '<script type="text/javascript">
		jQuery(document).ready(function($) {';

			if ($args['post_layout'] == 'slider') {
				$code .= '$("#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'slider").flexslider({
					animation: "fade",';

				if (isset($args[ 'show_nav' ]) && $args[ 'show_nav' ]) {
					$code .= 'directionNav: true,';
				} else {
					$code .= 'directionNav: false,';
				}

				$code .= '});';
			}

		$code .= '});
		</script>';
	}

	// Styling

	if ( !isset($args['color']) || $args['color'] == '') {
		$args['color'] = '#000000';
	}

	list($r, $g, $b)                               = sscanf($args['color'], "#%02x%02x%02x");
	list($text_r, $text_g, $text_b)                = sscanf($args['text_color'], "#%02x%02x%02x");

	$style = '';

	if ($args['style_override']) {

		$style .= '<style>

			/* Background Color */

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tile-item {
				background-color: rgba(' . $r . ', ' . $g . ', ' . $b . ', 0.8);
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'post-title {
				background-color: rgba(' . $r . ', ' . $g . ', ' . $b . ', 0.9);
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'colored-tabs-item {
				background-color: rgb(' . $r . ', ' . $g . ', ' . $b . ');
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-item {
				background-color: rgb(' . $r . ', ' . $g . ', ' . $b . ');
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-image .' . NNRobots_Post_Author::$prefix_dash . 'post-title,
			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-image .' . NNRobots_Post_Author::$prefix_dash . 'meta-data {
				background-color: rgba(' . $r . ', ' . $g . ', ' . $b . ', 0.5);
			}

			/* Text Color */

			#' . $this_id . ' a,
			#' . $this_id . ' p {
				color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'list li a {
				color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'list li span {
				color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'thumbnail li a {
				color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'thumbnail li span {
				color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'post-title {
				color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'comment-count,
			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'meta-data *  {
				color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'fancy-colors-content .' . NNRobots_Post_Author::$prefix_dash . 'post-title,
			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'fancy-colors-content .' . NNRobots_Post_Author::$prefix_dash . 'comment-count,
			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'fancy-colors-content .' . NNRobots_Post_Author::$prefix_dash . 'meta-data {
				color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'colored-tabs-content .' . NNRobots_Post_Author::$prefix_dash . 'post-title,
			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'colored-tabs-content .' . NNRobots_Post_Author::$prefix_dash . 'comment-count,
			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'colored-tabs-content .' . NNRobots_Post_Author::$prefix_dash . 'meta-data {
				color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-item a,
			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-content .' . NNRobots_Post_Author::$prefix_dash . 'post-title,
			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-item .' . NNRobots_Post_Author::$prefix_dash . 'comment-count,
			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-item .' . NNRobots_Post_Author::$prefix_dash . 'meta-data {
				color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
			}

		</style>';

	} else {

		$style .= '<style>

			/* Background Color */

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tile-item {
				background-color: rgba(0, 0, 0, 0.8);
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-item {
				background-color: white;
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'colored-tabs-item {
				background-color: white;
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'post-title {
				background-color: rgba(0, 0, 0, 0.9);
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-image .' . NNRobots_Post_Author::$prefix_dash . 'post-title,
			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-image .' . NNRobots_Post_Author::$prefix_dash . 'meta-data {
				background-color: rgba(0, 0, 0, 0.5);
			}

			/* Text Color */

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'post-title {
				color: white;
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'comment-count,
			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'meta-data *  {
				color: white;
			}

			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-content a,
			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-content .' . NNRobots_Post_Author::$prefix_dash . 'post-title,
			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-content .' . NNRobots_Post_Author::$prefix_dash . 'comment-count,
			#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-content .' . NNRobots_Post_Author::$prefix_dash . 'meta-data {
				color: white;
			}

		</style>';
	}

	wp_enqueue_style(NNRobots_Post_Author::$prefix . 'css', plugins_url('core/css/author-posts.css', __FILE__));

	return $code . $style;
}