<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

// Activate Widget

add_action('widgets_init', 	array('NNRobots_Post_Author_Widget', 'register_widget'));

// AJAX

add_action( 'wp_ajax_nnr_ap_display', 			array('NNRobots_Post_Author_Widget', 'display'));
add_action( 'wp_ajax_nopriv_nnr_ap_display', 	array('NNRobots_Post_Author_Widget', 'display'));

/**
 * NNRobots_Post_Author_Widget class.
 *
 * @extends WP_Widget
 */
class NNRobots_Post_Author_Widget extends WP_Widget {

	/**
	 * Register widget
	 *
	 * @since 1.0.0
	 *
	 * @param 	N/A
	 * @return 	N/A
	 */
	static function register_widget() {
		register_widget( 'NNRobots_Post_Author_Widget' );
	}

	/**
	 * Register widget with WordPress.
	 */
	function __construct() {
		parent::__construct(
			NNRobots_Post_Author::$prefix, 																					// Base ID
			__('Author Posts' , NNRobots_Post_Author::$text_domain), 														// Name
			array( 'description' => __( 'Display of list of all posts by an author.', NNRobots_Post_Author::$text_domain)) 	// Args
		);
	}

	/**
	 * Display all the content for Posts by Author via AJAX
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function display() {

		// Convert all strings to bool when needed

		if ( isset($_POST['instance']['async']) && is_string($_POST['instance']['async']) &&
			( $_POST['instance']['async'] == 'true' || $_POST['instance']['async'] == 'on' ) ) {
			$_POST['instance']['async'] = true;
		} else {
			$_POST['instance']['async'] = false;
		}

		if ( !isset($_POST['author_ID']) ) {
			$_POST['author_ID'] = 0;
		}

		// Pass the data to the display function

		if (isset($_POST['shortcode']) && $_POST['shortcode']) {
			echo json_encode(array('content' => self::display_posts_by_author($_POST['instance'], $_POST['post'], true, $_POST['author_ID'], $_POST['widget_id']), 'widget_id' => $_POST['widget_id']));
		} else {
			echo json_encode(array('content' => self::display_posts_by_author($_POST['instance'], $_POST['post'], false, $_POST['author_ID'], $_POST['widget_id']), 'widget_id' => $_POST['widget_id']));
		}

		die();

	}

	/**
	 * Output the HTML
	 *
	 * @access public
	 * @static
	 * @param mixed $instance
	 * @param bool $shortcode (default: false)
	 * @return void
	 */
	static function display_posts_by_author($instance, $post, $shortcode = false, $author_ID = 0, $this_id = 0){

		// Convert post object or string to array

		if ( is_object($post) ) {
			$post = (array) $post;
		}

		if ( !is_array($post) && is_string($post) ) {
			$post = (array) json_decode($post);
		}

		// Styling

		if ( !isset($instance['color']) || $instance['color'] == '') {
			$instance['color'] = '#000000';
		}

		list($r, $g, $b)                               = sscanf($instance['color'], "#%02x%02x%02x");
		list($text_r, $text_g, $text_b)                = sscanf($instance['text_color'], "#%02x%02x%02x");

		$style = '';

		if ($instance['style_override']) {

			$style .= '<style>

				/* Background Color */

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tile-item {
					background-color: rgba(' . $r . ', ' . $g . ', ' . $b . ', 0.8);
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'post-title {
					background-color: rgba(' . $r . ', ' . $g . ', ' . $b . ', 0.9);
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'colored-tabs-item {
					background-color: rgb(' . $r . ', ' . $g . ', ' . $b . ');
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-item {
					background-color: rgb(' . $r . ', ' . $g . ', ' . $b . ');
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-image .' . NNRobots_Post_Author::$prefix_dash . 'post-title,
				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-image .' . NNRobots_Post_Author::$prefix_dash . 'meta-data {
					background-color: rgba(' . $r . ', ' . $g . ', ' . $b . ', 0.5);
				}

				/* Text Color */

				#' . $this_id . ' a,
				#' . $this_id . ' p {
					color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'list li a {
					color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'list li span {
					color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'thumbnail li a {
					color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'thumbnail li span {
					color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'post-title {
					color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'comment-count,
				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'meta-data *  {
					color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'fancy-colors-content .' . NNRobots_Post_Author::$prefix_dash . 'post-title,
				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'fancy-colors-content .' . NNRobots_Post_Author::$prefix_dash . 'comment-count,
				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'fancy-colors-content .' . NNRobots_Post_Author::$prefix_dash . 'meta-data {
					color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'colored-tabs-content .' . NNRobots_Post_Author::$prefix_dash . 'post-title,
				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'colored-tabs-content .' . NNRobots_Post_Author::$prefix_dash . 'comment-count,
				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'colored-tabs-content .' . NNRobots_Post_Author::$prefix_dash . 'meta-data {
					color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-item a,
				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-content .' . NNRobots_Post_Author::$prefix_dash . 'post-title,
				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-item .' . NNRobots_Post_Author::$prefix_dash . 'comment-count,
				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-item .' . NNRobots_Post_Author::$prefix_dash . 'meta-data {
					color: rgb(' . $text_r . ', ' . $text_g . ', ' . $text_b . ');
				}

			</style>';

		} else {

			$style .= '<style>

				/* Background Color */

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tile-item {
					background-color: rgba(0, 0, 0, 0.8);
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-item {
					background-color: white;
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'colored-tabs-item {
					background-color: white;
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'post-title {
					background-color: rgba(0, 0, 0, 0.9);
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-image .' . NNRobots_Post_Author::$prefix_dash . 'post-title,
				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-image .' . NNRobots_Post_Author::$prefix_dash . 'meta-data {
					background-color: rgba(0, 0, 0, 0.5);
				}

				/* Text Color */

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'post-title {
					color: white;
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'comment-count,
				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-tiles .' . NNRobots_Post_Author::$prefix_dash . 'meta-data *  {
					color: white;
				}

				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-content a,
				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-content .' . NNRobots_Post_Author::$prefix_dash . 'post-title,
				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-content .' . NNRobots_Post_Author::$prefix_dash . 'comment-count,
				#' . $this_id . ' .' . NNRobots_Post_Author::$prefix_dash . 'image-flip-content .' . NNRobots_Post_Author::$prefix_dash . 'meta-data {
					color: white;
				}

			</style>';
		}

		$content = $style;

		$title                = isset($instance['title']) ? $instance['title'] : __('Author Posts', NNRobots_Post_Author::$text_domain);
		$orderby              = isset($instance['orderby']) ? $instance['orderby'] : 'ID';
		$order                = isset($instance['order']) ? $instance['order'] : 'ASC';
		$async                = isset($instance['async']) ? $instance['async'] : false;
		$num_posts            = isset($instance['num_posts']) ? $instance['num_posts'] : '5';
		$avatar_shape         = isset($instance['avatar_shape']) ? $instance['avatar_shape'] : 'rounded-square';
		$avatar_size          = isset($instance['avatar_size']) ? $instance['avatar_size'] : 'medium';
		$author_layout        = isset($instance['author_layout']) ? $instance['author_layout'] : 'inline';
		$author_description   = isset($instance['author_description']) ? $instance['author_description'] : false;
		$post_layout   		  = isset($instance['post_layout']) ? $instance['post_layout'] : 'list';

		if (!isset($num_posts)) {
			$num_posts = 'all';
		}

		if ( $author_ID == 0 ) {
			$author_ID = $post['post_author'];
		}

		$posts = get_posts(array(
			'posts_per_page'   => $num_posts != 'all' ? $num_posts : -1,
			'offset'           => 0,
			'post__not_in'	   => array($post['ID']),
			'orderby'          => $orderby,
			'order'            => $order,
			'post_type'        => 'post',
			'post_status'      => 'publish',
			'suppress_filters' => true,
			'author'		   => $author_ID
		));

		$user_data = get_userdata($author_ID);

		$container_class = '';

		if ( $shortcode ) {
			$container_class = NNRobots_Post_Author::$prefix_dash . 'shortcode';
		} else {
			$container_class = NNRobots_Post_Author::$prefix_dash . 'widget';
		}

		if ( $user_data ) {
			$content .= '<div class="' . $container_class . '" ' . ($shortcode ?  'id="' . $this_id . '"' : '') . '>';

				if ( $shortcode ) {
					$content .= '<h3 class="' . NNRobots_Post_Author::$prefix_dash . 'shortcode-title">' . $title . '</h3>';
				}

				$content .= '<div class="' . NNRobots_Post_Author::$prefix_dash . 'author-container ' . NNRobots_Post_Author::$prefix_dash . 'author-container-' . $author_layout . '">

					<a class="' . NNRobots_Post_Author::$prefix_dash . 'author-avatar-container" href="' . get_author_posts_url($author_ID) . '">
						<div class="' . NNRobots_Post_Author::$prefix_dash . 'author-avatar ' . NNRobots_Post_Author::$prefix_dash . 'author-avatar-' . $avatar_size . ' ' . NNRobots_Post_Author::$prefix_dash . 'author-avatar-' . $avatar_shape . '">';

						$content .= get_avatar($user_data->ID, 300);

						$content .= '</div>
					</a>';

					if ( $shortcode ) {
						$content .= '<h2 class="' . NNRobots_Post_Author::$prefix_dash . 'author-link">' . __($user_data->nickname, NNRobots_Post_Author::$text_domain) . '</h2>';
					} else {
						$content .= '<a class="' . NNRobots_Post_Author::$prefix_dash . 'author-link" href="' . get_author_posts_url($author_ID) . '">' . __($user_data->nickname, NNRobots_Post_Author::$text_domain) . '</a>';
					}

					if ( $author_description ) {
						$content .= '<p class="' . NNRobots_Post_Author::$prefix_dash . 'author-description">' . __($user_data->description, NNRobots_Post_Author::$text_domain) . '</p>';
					}

				$content .= '</div>

				<div class="' . NNRobots_Post_Author::$prefix_dash . 'author-posts-container">';

				switch ($post_layout) {

					case 'list_excerpt':

						$content .= '<ul class="' . NNRobots_Post_Author::$prefix_dash . 'author-post-list">';

						foreach ($posts as $item) {
							if ($item->ID != $post['ID']) {
								$content .= '<li class="' . NNRobots_Post_Author::$prefix_dash . 'author-post-list-item">
									<a class="' . NNRobots_Post_Author::$prefix_dash . 'post-title" href="' . get_permalink($item->ID) . '">' . $item->post_title . '</a>
									<p class="' . NNRobots_Post_Author::$prefix_dash . 'post-excerpt"><em>' .  self::get_post_excerpt($item) . '</em></p>
								</li>';
							}
						}

						$content .= '</ul>';

					break;

					case 'thumbnail':

						$content .= '<div class="' . NNRobots_Post_Author::$prefix_dash . 'author-post-thumbnail">';

						foreach ($posts as $item) {
							if ($item->ID != $post['ID']) {
								$content .= '<div class="' . NNRobots_Post_Author::$prefix_dash . 'author-post-thumbnail-item">
									<div class="' . NNRobots_Post_Author::$prefix_dash . 'thumbnail-image-container">' .get_the_post_thumbnail( $item->ID, 'thumbnail', array('class'	=> NNRobots_Post_Author::$prefix_dash . 'thumbnail-image')) . '</div>
									<a class="' . NNRobots_Post_Author::$prefix_dash . 'post-title" href="' . get_permalink($item->ID) . '">' . $item->post_title . '</a>
								</div>';
							}
						}

						$content .= '</div>';

					break;

					case 'numbered_list':

						$content .= '<ol class="' . NNRobots_Post_Author::$prefix_dash . 'author-post-numbered-list">';

						foreach ($posts as $item) {
							if ($item->ID != $post['ID']) {
								$content .= '<li class="' . NNRobots_Post_Author::$prefix_dash . 'author-post-list-item">
									<a class="' . NNRobots_Post_Author::$prefix_dash . 'post-title" href="' . get_permalink($item->ID) . '">' . $item->post_title . '</a>
								</li>';
							}
						}

						$content .= '</ol>';

					break;

					case 'image_tiles':

						$content .= '<div class="' . NNRobots_Post_Author::$prefix_dash . 'image-tiles">';

						foreach ($posts as $item) {

							$content .= '<div class="' . NNRobots_Post_Author::$prefix_dash . 'image-tile-item">' .
									get_the_post_thumbnail( $item->ID, 'large') .
								'<div class="' . NNRobots_Post_Author::$prefix_dash . 'post-title"><a href="' . get_permalink($item->ID) . '">' . $item->post_title . '</a></div>
							</div>';

						}

						$content .= '</div>';

					break;

					case 'image_flip':

						$content .= '<div class="' . NNRobots_Post_Author::$prefix_dash . 'image-flip">';

						foreach ($posts as $item) {

							$image = wp_get_attachment_image_src(get_post_thumbnail_id($item->ID), 'large');

							if ( $image ) {

								$content .= '<div class="' . NNRobots_Post_Author::$prefix_dash . 'image-flip-item">
									<div class="' . NNRobots_Post_Author::$prefix_dash . 'image-flip-content">' .
										'<a class="' . NNRobots_Post_Author::$prefix_dash . 'post-title">' . $item->post_title . '</a>' .
									'</div>
									<div class="' . NNRobots_Post_Author::$prefix_dash . 'image-flip-image" style="background-image: url(' . $image[0] . ');">
										<a class="' . NNRobots_Post_Author::$prefix_dash . 'post-title" href="' . get_permalink($item->ID) . '">' . $item->post_title . '</a>
									</div>
								</div>';

							}

						}

						$content .= '</div>';

					break;

					case 'grid':

						$content .= '<div class="' . NNRobots_Post_Author::$prefix_dash . 'grid ' . NNRobots_Post_Author::$prefix_dash . 'grid-' . $instance['grid_size'] . '">';

						foreach ($posts as $item) {

							if ($shortcode) {
								$image = wp_get_attachment_image_src(get_post_thumbnail_id($item->ID), 'large');
							} else {
								$image = wp_get_attachment_image_src(get_post_thumbnail_id($item->ID));
							}

							if ($image) {
								$content .= '<div class="' . NNRobots_Post_Author::$prefix_dash . 'grid-item">
									<a class="' . NNRobots_Post_Author::$prefix_dash . 'post-title" href="' . get_permalink($item->ID) . '" title="' . $item->post_title . '" style="background-image: url(' . $image[0] . ');"></a>
								</div>';
							}

						}

						$content .= '</div>';

					break;

					case 'slider':

						$content .= '<div class="' . NNRobots_Post_Author::$prefix_dash . 'slider flexslider"><ul class="slides">';

						foreach ($posts as $item) {

							$image = wp_get_attachment_image_src(get_post_thumbnail_id($item->ID), 'large');

							if ($image) {
								$content .= '<li><img alt="' . $item->post_title . '" src="' . $image[0] . '" />';
								$content .= '<div class="image-caption">
									<a class="' . NNRobots_Post_Author::$prefix_dash . 'post-title" href="' . get_permalink($item->ID) . '">' . $item->post_title . '</a>
								</div>';
								$content .= '</li>';
							}

						}

						$content .= '</ul></div>';

					break;

					case 'fancy_colors':

						$content .= '<div class="' . NNRobots_Post_Author::$prefix_dash . 'fancy-colors">';

						$fancy_colors = isset($instance['fancy_colors']) && $instance['fancy_colors'] != '' ? explode(',', $instance['fancy_colors']) : explode(',', '#D14841,#F37934,#FAC51C,#61BD6D,#2C82C9,#553982');

						$num_of_posts = 0;

						foreach ($posts as $item) {

							$color = $fancy_colors[$num_of_posts%count($fancy_colors)];

							$content .= '<div class="' . NNRobots_Post_Author::$prefix_dash . 'fancy-colors-content" style="background-color: ' . $color . ';">
								<div class="' . NNRobots_Post_Author::$prefix_dash . 'post-title"><a href="' . get_permalink($item->ID) . '">' . $item->post_title . '</a></div>
							</div>';

							$num_of_posts++;

						}

						$content .= '</div>';

					break;

					case 'colored_tabs':

						$content .= '<div class="' . NNRobots_Post_Author::$prefix_dash . 'colored-tabs">';

						$fancy_colors = isset($instance['fancy_colors']) && $instance['fancy_colors'] != '' ? explode(',', $instance['fancy_colors']) : explode(',', '#D14841,#F37934,#FAC51C,#61BD6D,#2C82C9,#553982');

						$num_of_posts = 0;

						foreach ($posts as $item) {

							$color = $fancy_colors[$num_of_posts%count($fancy_colors)];

							$content .= '<div class="' . NNRobots_Post_Author::$prefix_dash . 'colored-tabs-item">
								<div class="' . NNRobots_Post_Author::$prefix_dash . 'colored-tabs-color" style="background-color: ' . $color . ';"></div>
								<div class="' . NNRobots_Post_Author::$prefix_dash . 'colored-tabs-content">
									<div class="' . NNRobots_Post_Author::$prefix_dash . 'post-title"><a href="' . get_permalink($item->ID) . '">' . $item->post_title . '</a></div>
								</div>
							</div>';

							$num_of_posts++;

						}

						$content .= '</div>';

					break;

					default: // list

						$content .= '<ul class="' . NNRobots_Post_Author::$prefix_dash . 'author-post-list">';

						foreach ($posts as $item) {
							if ($item->ID != $post['ID']) {
								$content .= '<li class="' . NNRobots_Post_Author::$prefix_dash . 'author-post-list-item">
									<a class="' . NNRobots_Post_Author::$prefix_dash . 'post-title" href="' . get_permalink($item->ID) . '">' . $item->post_title . '</a>
								</li>';
							}
						}

						$content .= '</ul>';

					break;
				}

				$content .= '</div>
			</div>';
		}

		return $content;

	}

	/**
	 * Front-end display of widget.
	 *
	 * @see WP_Widget::widget()
	 *
	 * @param array $args     Widget arguments.
	 * @param array $instance Saved values from database.
	 */
	public function widget( $args, $instance ) {

		// Only show on posts with an author

		if ( !is_singular() ) {
			return;
		}

		// Do not show on pages

		if ( isset($instance['show_on_pages']) && !$instance['show_on_pages'] && is_singular('page') ) {
			return;
		}

		echo $args['before_widget'];

		$title = isset($instance['title']) ? $instance['title'] : '';

		if ( ! empty( $title ) ) {
			echo $args['before_title'] . $title . $args['after_title'];
		}

		global $post;

		// AJAX

		if ( isset($instance['async']) && $instance['async'] ) {

			echo '<script type="text/javascript">
			jQuery(document).ready(function($) {

				var data = {
					"action": "nnr_ap_display",
					"widget_id" : "' . $this->id . '",
					"instance" : ' . json_encode($instance) . ',
					"post": ' . json_encode($post) . ',
				};

				$.post("' . admin_url( 'admin-ajax.php' ) . '", data, function(response) {
					response = jQuery.parseJSON(response);
					$("#" + response.widget_id).append(response.content).hide().fadeIn();';

					if ($instance['post_layout'] == 'slider') {
						echo '$("#" + response.widget_id + " .' . NNRobots_Post_Author::$prefix_dash . 'slider").flexslider({
							animation: "fade",';

						if (isset($instance[ 'show_nav' ]) && $instance[ 'show_nav' ]) {
							echo 'directionNav: true,';
						} else {
							echo 'directionNav: false,';
						}

						echo '});';
					}

				echo '});
			});
			</script>';
		} else {
			echo self::display_posts_by_author($instance, $post, false, 0, $this->id);
			echo '<script type="text/javascript">
			jQuery(document).ready(function($) {';

				if ($instance['post_layout'] == 'slider') {
					echo '$("#' . $this->id . ' .' . NNRobots_Post_Author::$prefix_dash . 'slider").flexslider({
						animation: "fade",';

					if (isset($instance[ 'show_nav' ]) && $instance[ 'show_nav' ]) {
						echo 'directionNav: true,';
					} else {
						echo 'directionNav: false,';
					}

					echo '});';
				}

			echo '});
			</script>';
		}

		// Include slider files

		if ($instance['post_layout'] == 'slider') {
			wp_enqueue_script(NNRobots_Post_Author::$prefix . 'slider_js', plugins_url('core/includes/flex-slider/jquery.flexslider-min.js', __FILE__), array('jquery'));
			wp_enqueue_style(NNRobots_Post_Author::$prefix . 'slider_css', plugins_url('core/includes/flex-slider/flexslider.css', __FILE__));
		}

		wp_enqueue_style(NNRobots_Post_Author::$prefix . 'css', plugins_url('core/css/author-posts.css', __FILE__));

		echo $args['after_widget'];
	}

	/**
	 * Back-end widget form.
	 *
	 * @see WP_Widget::form()
	 *
	 * @param array $instance Previously saved values from database.
	 */
	public function form( $instance ) {

		isset($instance[ 'title' ]) ? $title = $instance[ 'title' ] : $title = __('Author Posts', NNRobots_Post_Author::$text_domain);
		isset($instance[ 'orderby' ]) ? $orderby = $instance[ 'orderby' ] : $orderby = '';
		isset($instance[ 'order' ]) ? $order = $instance[ 'order' ] : $order = '';
		isset($instance[ 'async' ]) ? $async = $instance[ 'async' ] : $async = '';
		isset($instance[ 'show_on_pages' ]) ? $show_on_pages = $instance[ 'show_on_pages' ] : $show_on_pages = false;
		isset($instance[ 'num_posts' ]) ? $num_posts = $instance[ 'num_posts' ] : $num_posts = '';
		isset($instance[ 'avatar_shape' ]) ? $avatar_shape = $instance[ 'avatar_shape' ] : $avatar_shape = '';
		isset($instance[ 'avatar_size' ]) ? $avatar_size = $instance[ 'avatar_size' ] : $avatar_size = '';
		isset($instance[ 'author_layout' ]) ? $author_layout = $instance[ 'author_layout' ] : $author_layout = '';
		isset($instance[ 'author_description' ]) ? $author_description = $instance[ 'author_description' ] : $author_description = '';
		isset($instance[ 'post_layout' ]) ? $post_layout = $instance[ 'post_layout' ] : $post_layout = '';

		isset($instance[ 'grid_size' ]) ? $grid_size = $instance[ 'grid_size' ] : $grid_size = '';
		isset($instance[ 'show_nav' ]) ? $show_nav = $instance[ 'show_nav' ] : $show_nav = '';
		isset($instance[ 'fancy_colors' ]) ? $fancy_colors = $instance[ 'fancy_colors' ] : $fancy_colors = '#D14841,#F37934,#FAC51C,#61BD6D,#2C82C9,#553982';

		isset($instance[ 'style_override' ]) ? $style_override = $instance[ 'style_override' ] : $style_override = '';
		isset($instance[ 'color' ]) ? $color = $instance[ 'color' ] : $color = '';
		isset($instance[ 'text_color' ]) ? $text_color = $instance[ 'text_color' ] : $text_color = '';

		wp_enqueue_style( 'wp-color-picker');
		wp_enqueue_script( 'wp-color-picker');

		wp_enqueue_script(NNRobots_Post_Author::$prefix . 'widget_js', plugins_url('admin/js/widget-settings.js', __FILE__), array('jquery', 'wp-color-picker'), true, false);
		wp_localize_script(NNRobots_Post_Author::$prefix . 'widget_js', 'nnr_ap_widget_settings', array(
			'prefix_dash'	=> NNRobots_Post_Author::$prefix_dash,
		));

		wp_enqueue_style(NNRobots_Post_Author::$prefix . 'widget-css', plugins_url('admin/css/widget-settings.css', __FILE__));

		$id_prefix = $this->id . NNRobots_Post_Author::$prefix_dash;

		require('admin/views/post-by-author-widget-settings.php');
	}

	/**
	 * Sanitize widget form values as they are saved.
	 *
	 * @see WP_Widget::update()
	 *
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 *
	 * @return array Updated safe values to be saved.
	 */
	public function update( $new_instance, $old_instance ) {

		$instance = array();
		$instance['title']                = ( ! empty( $new_instance['title'] ) ) ? sanitize_text_field( $new_instance['title'] ) : '';
		$instance['orderby']              = ( ! empty( $new_instance['orderby'] ) ) ? sanitize_text_field( $new_instance['orderby'] ) : '';
		$instance['order']                = ( ! empty( $new_instance['order'] ) ) ? sanitize_text_field( $new_instance['order'] ) : '';
		$instance['async']                = ( ! empty( $new_instance['async'] ) ) ? sanitize_text_field( $new_instance['async'] ) : '';
		$instance['show_on_pages']        = ( ! empty( $new_instance['show_on_pages'] ) ) ? sanitize_text_field( $new_instance['show_on_pages'] ) : '';
		$instance['num_posts']            = ( ! empty( $new_instance['num_posts'] ) ) ? sanitize_text_field( $new_instance['num_posts'] ) : '';
		$instance['avatar_shape']         = ( ! empty( $new_instance['avatar_shape'] ) ) ? sanitize_text_field( $new_instance['avatar_shape'] ) : '';
		$instance['avatar_size']          = ( ! empty( $new_instance['avatar_size'] ) ) ? sanitize_text_field( $new_instance['avatar_size'] ) : '';
		$instance['author_layout']        = ( ! empty( $new_instance['author_layout'] ) ) ? sanitize_text_field( $new_instance['author_layout'] ) : '';
		$instance['author_description']   = ( ! empty( $new_instance['author_description'] ) ) ? sanitize_text_field( $new_instance['author_description'] ) : '';
		$instance['post_layout']          = ( ! empty( $new_instance['post_layout'] ) ) ? sanitize_text_field( $new_instance['post_layout'] ) : '';

		$instance['grid_size']            = ( ! empty( $new_instance['grid_size'] ) ) ? sanitize_text_field( $new_instance['grid_size'] ) : '';
		$instance['show_nav']             = ( ! empty( $new_instance['show_nav'] ) ) ? sanitize_text_field( $new_instance['show_nav'] ) : '';
		$instance['fancy_colors']         = ( ! empty( $new_instance['fancy_colors'] ) ) ? sanitize_text_field( $new_instance['fancy_colors'] ) : '';

		$instance['style_override']       = ( ! empty( $new_instance['style_override'] ) ) ? sanitize_text_field( $new_instance['style_override'] ) : '';
		$instance['color']         		  = ( ! empty( $new_instance['color'] ) ) ? sanitize_text_field( $new_instance['color'] ) : '';
		$instance['text_color']        	  = ( ! empty( $new_instance['text_color'] ) ) ? sanitize_text_field( $new_instance['text_color'] ) : '';

		return $instance;
	}

	/**
	 * Get the excerpt of the post
	 *
	 * @access public
	 * @param mixed $post
	 * @return void
	 */
	public static function get_post_excerpt( $post ) {

		$excerpt = $post->post_excerpt;

		if ( !isset($excerpt) || empty($excerpt) || $excerpt == '') {
			$excerpt = $post->post_content;
		}

		return wp_trim_words( strip_shortcodes( strip_tags( $excerpt ) ), 35);
	}
}