<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

// Activate Widget

add_action('widgets_init', 	array('NNRobots_Featured_Author', 'register_widget'));

/**
 * NNRobots_Featured_Author class.
 *
 * @extends WP_Widget
 */
class NNRobots_Featured_Author extends WP_Widget {

	/**
	 * prefix_dash
	 *
	 * (default value: 'nnr-fa-')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $prefix_dash = 'nnr-fa-';

	/**
	 * Register widget with WordPress.
	 */
	static function register_widget() {
	    register_widget( 'NNRobots_Featured_Author' );
	}

	/**
	 * Register widget with WordPress.
	 */
	function __construct() {
		parent::__construct(
			'nnrobots_featured_author', // Base ID
			__('Featured Author', NNRobots_Post_Author::$text_domain), // Name
			array( 'description' => __( 'Displays a featured author.', NNRobots_Post_Author::$text_domain ), ) // Args
		);
	}

	/**
	 * Front-end display of widget.
	 *
	 * @see WP_Widget::widget()
	 *
	 * @param array $args     Widget arguments.
	 * @param array $instance Saved values from database.
	 */
	public function widget( $args, $instance ) {

		$title                = isset($instance['title']) ? $instance['title'] : __('Featured Author', NNRobots_Post_Author::$text_domain);
		$author_ID            = isset($instance['author_ID']) ? $instance['author_ID'] : '';
		$orderby              = isset($instance['orderby']) ? $instance['orderby'] : '';
		$order                = isset($instance['order']) ? $instance['order'] : '';
		$size                 = isset($instance['size']) ? $instance['size'] : '';
		$async                = isset($instance['async']) ? $instance['async'] : '';
		$num_posts            = isset($instance['num_posts']) ? $instance['num_posts'] : '';
		$avatar_shape         = isset($instance['avatar_shape']) ? $instance['avatar_shape'] : '';
		$avatar_size          = isset($instance['avatar_size']) ? $instance['avatar_size'] : '';
		$author_layout        = isset($instance['author_layout']) ? $instance['author_layout'] : '';
		$author_description   = isset($instance['author_description']) ? $instance['author_description'] : '';
		$post_layout          = isset($instance['post_layout']) ? $instance['post_layout'] : '';

		if (!isset($num_posts)) $num_posts = 'all';

		echo $args['before_widget'];
		echo $args['before_title'] . $title . $args['after_title'];

		global $post;

		// AJAX

		if ( isset($instance['async']) && $instance['async'] ) {

			echo '<script type="text/javascript">
			jQuery(document).ready(function($) {

				var data = {
					"action": "nnr_ap_display",
					"widget_id" : "' . $this->id . '",
					"instance" : ' . json_encode($instance) . ',
					"post": ' . json_encode($post) . ',
					"author_ID": ' . $author_ID . ',
				};

				$.post("' . admin_url( 'admin-ajax.php' ) . '", data, function(response) {
					response = jQuery.parseJSON(response);
					$("#" + response.widget_id).append(response.content).hide().fadeIn();';

					if ($instance['post_layout'] == 'slider') {
						echo '$("#" + response.widget_id + " .' . NNRobots_Post_Author::$prefix_dash . 'slider").flexslider({
							animation: "fade",';

						if (isset($instance[ 'show_nav' ]) && $instance[ 'show_nav' ]) {
							echo 'directionNav: true,';
						} else {
							echo 'directionNav: false,';
						}

						echo '});';
					}

				echo '});
			});
			</script>';
		} else {

			include_once('author-posts-widget.php');

			echo NNRobots_Post_Author_Widget::display_posts_by_author($instance, $post, false, $author_ID, $this->id);
			echo '<script type="text/javascript">
			jQuery(document).ready(function($) {';

				if ($instance['post_layout'] == 'slider') {
					echo '$("#' . $this->id . ' .' . NNRobots_Post_Author::$prefix_dash . 'slider").flexslider({
						animation: "fade",';

					if (isset($instance[ 'show_nav' ]) && $instance[ 'show_nav' ]) {
						echo 'directionNav: true,';
					} else {
						echo 'directionNav: false,';
					}

					echo '});';
				}

			echo '});
			</script>';
		}

		// Include slider files

		if ($instance['post_layout'] == 'slider') {
			wp_enqueue_script(NNRobots_Post_Author::$prefix . 'slider_js', plugins_url('core/includes/flex-slider/jquery.flexslider-min.js', __FILE__), array('jquery'));
			wp_enqueue_style(NNRobots_Post_Author::$prefix . 'slider_css', plugins_url('core/includes/flex-slider/flexslider.css', __FILE__));
		}

		echo $args['after_widget'];

		wp_enqueue_style('nnrobots_author_posts_css', plugins_url('core/css/author-posts.css', __FILE__));
	}

	/**
	 * Back-end widget form.
	 *
	 * @see WP_Widget::form()
	 *
	 * @param array $instance Previously saved values from database.
	 */
	public function form( $instance ) {



		isset($instance[ 'title' ]) ? $title = $instance[ 'title' ] : $title = __('Featured Author', NNRobots_Post_Author::$text_domain);
		isset($instance[ 'author_ID' ]) ? $author_ID = $instance[ 'author_ID' ] : $author_ID = '';
		isset($instance[ 'orderby' ]) ? $orderby = $instance[ 'orderby' ] : $orderby = '';;
		isset($instance[ 'order' ]) ? $order = $instance[ 'order' ] : $order = '';
		isset($instance[ 'async' ]) ? $async = $instance[ 'async' ] : $async = '';
		isset($instance[ 'num_posts' ]) ? $num_posts = $instance[ 'num_posts' ] : $num_posts = '';
		isset($instance[ 'avatar_shape' ]) ? $avatar_shape = $instance[ 'avatar_shape' ] : $avatar_shape = '';
		isset($instance[ 'avatar_size' ]) ? $avatar_size = $instance[ 'avatar_size' ] : $avatar_size = '';
		isset($instance[ 'author_layout' ]) ? $author_layout = $instance[ 'author_layout' ] : $author_layout = '';
		isset($instance[ 'author_description' ]) ? $author_description = $instance[ 'author_description' ] : $author_description = '';
		isset($instance[ 'post_layout' ]) ? $post_layout = $instance[ 'post_layout' ] : $post_layout = '';

		isset($instance[ 'grid_size' ]) ? $grid_size = $instance[ 'grid_size' ] : $grid_size = '';
		isset($instance[ 'show_nav' ]) ? $show_nav = $instance[ 'show_nav' ] : $show_nav = '';
		isset($instance[ 'show_nav' ]) ? $fancy_colors = $instance[ 'fancy_colors' ] : $fancy_colors = '';

		isset($instance[ 'style_override' ]) ? $style_override = $instance[ 'style_override' ] : $style_override = '';
		isset($instance[ 'color' ]) ? $color = $instance[ 'color' ] : $color = '';
		isset($instance[ 'text_color' ]) ? $text_color = $instance[ 'text_color' ] : $text_color = '';

		wp_enqueue_style( 'wp-color-picker');
		wp_enqueue_script( 'wp-color-picker');

		wp_enqueue_style('nnrobots_featured_author_selectize_css', 	plugins_url('admin/css/selectize.css', __FILE__));
		wp_enqueue_script('nnrobots_featured_author_selectize_js', 	plugins_url('admin/js/selectize.min.js', __FILE__));
		wp_enqueue_script('nnrobots_featured_author_js', 			plugins_url('admin/js/featured-author-widget-settings.js', __FILE__), array('jquery', 'wp-color-picker'), true, false);
		wp_localize_script('nnrobots_featured_author_js', 			'nnr_ap_fa_widget_settings', array(
			'prefix_dash'	=> NNRobots_Post_Author::$prefix_dash,
			'authors'		=> get_users(array('who' => 'authors', 'number' => 500)),
		));

		$id_prefix = $this->id . NNRobots_Post_Author::$prefix_dash;

		require('admin/views/featured-author-widget-settings.php');
	}

	/**
	 * Sanitize widget form values as they are saved.
	 *
	 * @see WP_Widget::update()
	 *
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 *
	 * @return array Updated safe values to be saved.
	 */
	public function update( $new_instance, $old_instance ) {

		$instance = array();

		$instance['title']                = ( ! empty( $new_instance['title'] ) ) ? sanitize_text_field( $new_instance['title'] ) : '';
		$instance['author_ID']            = ( ! empty( $new_instance['author_ID'] ) ) ? sanitize_text_field( $new_instance['author_ID'] ) : '';
		$instance['orderby']              = ( ! empty( $new_instance['orderby'] ) ) ? sanitize_text_field( $new_instance['orderby'] ) : '';
		$instance['order']                = ( ! empty( $new_instance['order'] ) ) ? sanitize_text_field( $new_instance['order'] ) : '';
		$instance['async']                = ( ! empty( $new_instance['async'] ) ) ? sanitize_text_field( $new_instance['async'] ) : '';
		$instance['num_posts']            = ( ! empty( $new_instance['num_posts'] ) ) ? sanitize_text_field( $new_instance['num_posts'] ) : '';
		$instance['avatar_shape']         = ( ! empty( $new_instance['avatar_shape'] ) ) ? sanitize_text_field( $new_instance['avatar_shape'] ) : '';
		$instance['avatar_size']          = ( ! empty( $new_instance['avatar_size'] ) ) ? sanitize_text_field( $new_instance['avatar_size'] ) : '';
		$instance['author_layout']        = ( ! empty( $new_instance['author_layout'] ) ) ? sanitize_text_field( $new_instance['author_layout'] ) : '';
		$instance['author_description']   = ( ! empty( $new_instance['author_description'] ) ) ? sanitize_text_field( $new_instance['author_description'] ) : '';
		$instance['post_layout']          = ( ! empty( $new_instance['post_layout'] ) ) ? sanitize_text_field( $new_instance['post_layout'] ) : '';

		$instance['grid_size']            = ( ! empty( $new_instance['grid_size'] ) ) ? sanitize_text_field( $new_instance['grid_size'] ) : '';
		$instance['show_nav']             = ( ! empty( $new_instance['show_nav'] ) ) ? sanitize_text_field( $new_instance['show_nav'] ) : '';
		$instance['fancy_colors']         = ( ! empty( $new_instance['fancy_colors'] ) ) ? sanitize_text_field( $new_instance['fancy_colors'] ) : '';

		$instance['style_override']       = ( ! empty( $new_instance['style_override'] ) ) ? sanitize_text_field( $new_instance['style_override'] ) : '';
		$instance['color']         		  = ( ! empty( $new_instance['color'] ) ) ? sanitize_text_field( $new_instance['color'] ) : '';
		$instance['text_color']        	  = ( ! empty( $new_instance['text_color'] ) ) ? sanitize_text_field( $new_instance['text_color'] ) : '';

		return $instance;
	}

}